<?php

namespace Drupal\twitter_api_block\Plugin\Block;

use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\UncacheableDependencyTrait;
use Drupal\Core\Form\FormStateInterface;
use TwitterAPIExchange;

/**
 * Provides a 'TwitterBlock' block.
 *
 * @Block(
 *   id = "twitter_api_block",
 *   admin_label = @Translation("Twitter block"),
 *   category = @Translation("Content")
 * )
 *
 * @see https://github.com/J7mbo/twitter-api-php
 */
class TwitterBlock extends BlockBase {

  use UncacheableDependencyTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form   = parent::blockForm($form, $form_state);
    $config = $this->getConfiguration();

    // Block options.
    $form['options'] = [
      '#type'        => 'details',
      '#title'       => $this->t('Options'),
      '#collapsible' => false,
      '#open'        => TRUE,
    ];
    $form['options']['username'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t("Username"),
      '#default_value' => isset($config['options']['username']) ? $config['options']['username'] : NULL,
      '#required'      => TRUE,
    ];
    $form['options']['count'] = [
      '#type'          => 'number',
      '#title'         => $this->t("Number of tweets"),
      '#default_value' => isset($config['options']['count']) ? $config['options']['count'] : 3,
      '#required'      => TRUE,
    ];
    $form['options']['replies'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t("Include replies"),
      '#default_value' => isset($config['options']['replies']) ? $config['options']['replies'] : 0,
    ];
    $form['options']['retweets'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t("Include retweets"),
      '#default_value' => isset($config['options']['retweets']) ? $config['options']['retweets'] : 0,
    ];
    // Twitter credentials.
    $form['api'] = [
      '#type'        => 'details',
      '#title'       => $this->t('Twitter credentials'),
      '#description' => $this->t('Create a new application at <a href="https://apps.twitter.com/" target="_blank">https://apps.twitter.com/</a>.'),
      '#collapsible' => TRUE,
      '#open'        => TRUE,
    ];
    $form['api']['consumer_key'] = [
      '#title'         => t('Consumer Key'),
      '#type'          => 'textfield',
      '#default_value' => isset($config['api']['consumer_key']) ? $config['api']['consumer_key'] : NULL,
      '#required'      => TRUE,
    ];
    $form['api']['consumer_secret'] = [
      '#title'         => t('Consumer Secret'),
      '#type'          => 'textfield',
      '#default_value' => isset($config['api']['consumer_secret']) ? $config['api']['consumer_secret'] : NULL,
      '#required'      => TRUE,
    ];
    $form['api']['oauth_access_token'] = [
      '#title'         => t('OAuth Access Token'),
      '#type'          => 'textfield',
      '#default_value' => isset($config['api']['oauth_access_token']) ? $config['api']['oauth_access_token'] : NULL,
      '#required'      => TRUE,
    ];
    $form['api']['oauth_access_token_secret'] = [
      '#title'         => t('OAuth Access Token Secret'),
      '#type'          => 'textfield',
      '#default_value' => isset($config['api']['oauth_access_token_secret']) ? $config['api']['oauth_access_token_secret'] : NULL,
      '#required'      => TRUE,
    ];
    // Cache-related.
    $form['options']['disable_cache'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t("Disable cache"),
      '#description'   => $this->t("" .
        "If checked, this block will trigger the `page_cache_kill_switch` when displayed." .
        "This has impact on your site's performance." .
        "Use with caution."),
      '#default_value' => isset($config['options']['replies']) ? $config['options']['replies'] : 0,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    // Save our custom settings when the form is submitted.
    foreach ($form_state->getValues() as $key => $value) {
      $this->setConfigurationValue($key, $value);
    };
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build  = [];
    $config = $this->getConfiguration();

    // Stop now if credentials are not setup.
    if (
      NULL == $config['api']['consumer_key'] ||
      NULL == $config['api']['consumer_secret'] ||
      NULL == $config['api']['oauth_access_token'] ||
      NULL == $config['api']['oauth_access_token_secret']
    ) {
      return [];
    }

    // Get credentials.
    $settings = [
      'consumer_key'              => $config['api']['consumer_key'],
      'consumer_secret'           => $config['api']['consumer_secret'],
      'oauth_access_token'        => $config['api']['oauth_access_token'],
      'oauth_access_token_secret' => $config['api']['oauth_access_token_secret'],
    ];

    // Get latest tweets.
    $tweets = [];
    try {
      $twitter  = new TwitterAPIExchange($settings);
      $username = $config['options']['username'];
      $url      = 'https://api.twitter.com/1.1/statuses/user_timeline.json';
      $params   = static::getParameters($config);
      $result   = $twitter->setGetfield($params)->buildOauth($url, 'GET')->performRequest();
      $latest   = Json::decode($result);

      $i = 0;
      foreach ($latest as $tweet) {
        if (isset($tweet['id']) && $i < $config['options']['count']) {
          $url      = 'https://publish.twitter.com/oembed?url=https%3A%2F%2Ftwitter.com%2F' . $username . '%2Fstatus%2F' . $tweet['id'];
          $data     = $twitter->buildOauth($url, 'GET')->performRequest();
          $tweets[] = Json::decode($data);
        }
        $i++;
      }
    } catch (\Exception $e) {
      \Drupal::logger('twitter')->error($e->getMessage());
    }

    // Build renderable array of oembed tweets.
    if (count($tweets)) {
      $build['tweets'] = [];
      foreach ($tweets as $tweet) {
        $build['tweets'][] = ['#markup' => $tweet['html']];
      }

      // Pass account name to Twig.
      $build['username'] = [
        '#type'   => 'item',
        '#markup' => $config['options']['username'],
      ];

      // Attach JS and styles.
      $build['#attached']['library'][] = 'twitter_api_block/base';
    }

    // Cache depending on the current user.
    $build['#cache'] = [
      'contexts' => [
        'user',
      ],
    ];

    if (isset($this->configuration['disable_cache']) && $this->configuration['disable_cache']) {
      // Do not cache any page with this block on it.
      \Drupal::service('page_cache_kill_switch')->trigger();

      // Disable caching.
      $build['#cache'] = [
        'contexts' => [
          'user',
        ],
        'max-age'  => 0,
      ];
    }

    return $build;
  }

  private static function getParameters($config) {
    return UrlHelper::buildQuery([
      'screen_name'     => $config['options']['username'],
      'count'           => $config['options']['count'],
      // Reverse exclude replies value on purpose.
      'exclude_replies' => ($config['options']['replies']) ? 'false' : 'true',
      'include_rts'     => $config['options']['retweets'] ? 'true' : 'false',
    ]);
  }
}
